// -----------------------------------------
// shuffle - Shuffle the contents of 'array'
// -----------------------------------------

function shuffle(array)
{
   var index = array.length;

   while(index > 0)
   {
      var i = Math.floor(Math.random() * index--);

      var tmp      = array[index];
      array[index] = array[i];
      array[i]     = tmp;
   }
}

// -----------------------------------------
// getRanks - Return an array of all
//            card ranks
// -----------------------------------------

function getRanks()
{
   return(
      [
         "two",   "three", "four", "five", "six",
         "seven", "eight", "nine", "ten",  "jack",
         "queen", "king",  "ace"
      ]
   );
}

// -----------------------------------------
// getDeck - Acquire our deck object
//           which is composed of 52 cards
//           and a 'next' method to deal
//           the next card.
// -----------------------------------------

function getDeck()
{
   var ranks = getRanks();
   var suits = [ "diamonds", "clubs", "hearts", "spades" ];
   var cards = [];

   for(var i = 0; i < ranks.length; i ++)
   {
      for(var j = 0; j < suits.length; j ++)
      {
         var card =
         {
            rank: ranks[i],
            suit: suits[j],
            name: ranks[i] + "-of-" + suits[j] + ".png",
            used: false
         };

         cards.push(card);
      }
   }

   shuffle(cards);

   var theObject =
   {
      next: function()
      {
         for(var i = 0; i < cards.length; i ++)
         {
            if(!cards[i].used)
            {
               cards[i].used = true;
               return(cards[i]);
            }
         }

         return(null);
      }
   };

   return(theObject);
}

// -----------------------------------------
// gid - Return the element with id 'name'
// -----------------------------------------

function gid(name)
{
   return(document.getElementById(name));
}

// -----------------------------------------
// showCard - Display the specified card
//            in element 'id'
// -----------------------------------------

function showCard(id, name)
{
   var element = gid(id);

   if(element != null)
   {
      var img = "<img src=\"images/" + name + "\">";
      element.innerHTML = img;
   }
}

// -----------------------------------------
// rankToNumber - Convert card rank to the
//                equivalent integer value
// -----------------------------------------

function rankToNumber(rank)
{
   return(getRanks().indexOf(rank) + 2);
}

// -----------------------------------------
// isFlush - Return true if the cards in
//           'list' are all the same suit
// -----------------------------------------

function isFlush(list)
{
   var count = 1;

   for(var i = 1; i < list.length; i ++)
      if(list[0].suit == list[i].suit)
         count++;

   return(count == list.length);
}

// -----------------------------------------
// getNumbers - Get the number value of all
//              cards in 'list' and sort
//              them from small to large
// -----------------------------------------

function getNumbers(list)
{
   var numbers = [];

   for(var i = 0; i < list.length; i ++)
      numbers.push(rankToNumber(list[i].rank));

   var compare = function(first, second) { return(first - second); };

   return(numbers.sort(compare));
}

// -----------------------------------------
// isStraight - Return true if each card's
//              number (when sorted) is one
//              more than the previous card
// -----------------------------------------

function isStraight(numbers)
{
   for(var i = 1; i < numbers.length; i ++)
      if(numbers[i] != numbers[i - 1] + 1)
         return(false);

   return(true);
}

// -----------------------------------------
// getHighCard - Return the value of the
//               highest-valued card in
//               'numbers'
// -----------------------------------------

function getHighCard(numbers)
{
   return(numbers[numbers.length - 1]);
}

// -----------------------------------------
// getGroups - Return an array of arrays
//             with each subarray consisting
//             of cards with the same number
//             (all 2's for example).
// -----------------------------------------

function getGroups(numbers)
{
   var groups = [];
   var i      = 0;

   while(i < numbers.length)
   {
      var group = [ numbers[i] ];

      for(var j = i + 1; j < numbers.length; j ++)
         if(numbers[j] == numbers[i])
            group.push(numbers[j]);

      groups.push(group);
      i += group.length;
   }

   return(groups);
}

// -----------------------------------------
// valueForCount - Return the rank (number)
//                 for the nth group with
//                 'count' elements.
// -----------------------------------------

function valueForCount(groups, count, n)
{
   var value = -1;

   for(var i = 0; value == -1 && i < groups.length; i ++)
      if(groups[i].length == count && --n == 0)
         value = groups[i][0];

   return(value);
}

// -----------------------------------------
// evaluateHand - Convert 'hand' to an int
//                value so we can compare
//                directly to other hands
//                to find the winner(s)
// -----------------------------------------

function evaluateHand(hand)
{
   var theObject =
   {
      description: "",
      value      : 0
   };

   var flush    = isFlush(hand);
   var numbers  = getNumbers(hand);
   var straight = isStraight(numbers);
   var high     = getHighCard(numbers);

   if(flush && straight)
   {
      if(high == rankToNumber("ace"))
      {
         theObject.description = "Royal Flush";
         theObject.value       = 900;
      }
      else
      {
         theObject.description = "Straight Flush";
         theObject.value       = 800 + high;
      }
   }
   else
   {
      var groups = getGroups(numbers);
      var v4     = valueForCount(groups, 4, 1);

      if(v4 != -1)
      {
         theObject.description = "Four Of A Kind";
         theObject.value       = 700 + v4;
      }
      else
      {
         var v3 = valueForCount(groups, 3, 1);
         var v2 = valueForCount(groups, 2, 1);

         if(v3 != -1 && v2 != -1)
         {
            theObject.description = "Full House";
            theObject.value       = 600 + (50 + v3) + v2;
         }
         else if(flush)
         {
            theObject.description = "Flush";
            theObject.value       = 500 + high;
         }
         else if(straight)
         {
            theObject.description = "Straight";
            theObject.value       = 400 + high;
         }
         else if(v3 != -1)
         {
            theObject.description = "Three Of A Kind";
            theObject.value       = 300 + v3;
         }
         else if(v2 != -1)
         {
            var other = valueForCount(groups, 2, 2);

            if(other != -1)
            {
               theObject.description = "Two Pair";
               theObject.value       = 200 + (50 + other) + v2;
            }
            else
            {
               theObject.description = "One Pair";
               theObject.value       = 100 + v2;
            }
         }
         else
         {
            theObject.description = "High Card";
            theObject.value       = high;
         }
      }
   }

   return(theObject);
}

// -----------------------------------------
// evaluate - Determine which player (or
//            players) is the winner and
//            display that result
// -----------------------------------------

function evaluate(playerToCards)
{
   var output =
      "<br>\n" +
      "<table border=0>" +
      "<tr>"   +
      "<td align='top'>" +
      "<table bgcolor='black' cellpadding='10' cellspacing='1'>";

   var list = [];

   for(var player in playerToCards)
   {
      var result = evaluateHand(playerToCards[player]);
      var name   = gid(player + "_name").innerHTML;

      list.push({ player: name, score: result.value });

      output += "<tr bgcolor='white'>";
      output += "<td><b>" + name + "</b></td>";
      output += "<td>" + result.description + "</td>";
      output += "</tr>";
   }

   output += "</table>";
   output += "</td>";
   output += "<td width='10%'>&nbsp;</td>";

   list.sort(
      function(a, b) { return(b.score - a.score); }
   );

   var n    = 1;
   var text = list[0].player;

   for(var i = 1; i < list.length; i ++)
   {
      if(list[i].score == list[0].score)
      {
         n++;
         text  += ", " + list[i].player;
      }
   }

   output +=
      "<td valign='top'>" +
       "<b class='winners'> Winner" +
        (n == 1 ? "" : "s") + ": " + text + "</b>";

   output += "</td></tr></table>";

   var r = gid("the-results");

   r.innerHTML = output;
}

// -----------------------------------------
// play - Play a round of poker
// -----------------------------------------

function play()
{
   var theDeck       = getDeck();
   var playerToCards = {};

   // -------------------------------
   // Find the number of players
   // -------------------------------

   var nPlayers = 0;

   for(;;)
   {
      var playerName = "player" + (nPlayers + 1);

      var id = playerName + "_1";

      if(gid(id) == null)
         break;

      playerToCards[playerName] = [];

      nPlayers++;
   }

   // -------------------------------
   // And the number of cards
   // -------------------------------

   var nCards = 0;

   for(;;)
   {
      var id = "player1" + "_" + (nCards + 1);

      if(gid(id) == null)
         break;

      nCards++;
   }

   // -------------------------------
   // Deal 'nCards' to each of the
   // 'nPlayers' and determine the
   // winner (or winners)
   // -------------------------------

   for(var i = 1; i <= nCards; i ++)
   {
      for(var j = 1; j <= nPlayers; j ++)
      {
         var base   = "player" + j;
         var player = base + "_" + i;
         var card   = theDeck.next();

         showCard(player, card.name);
         playerToCards[base].push(card);
      }
   }

   evaluate(playerToCards);
}
